# 
# Copyright 2018 baidu.com, Inc. or its affiliates. All Rights Reserved.
# 

import requests
import json
import commentjson
import re

from os.path import abspath, isfile, dirname
import sys
from urllib import urlencode

url = 'https://openapi.baidu.com/oauth/2.0/token?grant_type=client_credentials&'
body = {}
headers = {}

# Static JSON key for the config file.
authDelegateKey = 'authDelegate'



# JSON keys for config file
CLIENT_ID = 'clientId'
CLIENT_SECRET = 'clientSecret'
REFRESH_TOKEN = 'refreshToken'

# Read the configuration filename from the command line arguments -if it exists.
if 2 == len(sys.argv):
    configFilename = abspath(sys.argv[1])
else:
    print 'Please input configure file name.'

# Check if the configuration file exists.
if not isfile(configFilename):
    print 'The file "' + \
            configFilename + \
            '" does not exists. Please create this file and fill required data.'
    sys.exit(1)

try:
    configFile = open(configFilename,'r')

except IOError:
    print 'File "' + configFilename + '" not found!'
    sys.exit(1)
else:
    with configFile:
        configData = commentjson.load(configFile)
        if not configData.has_key(authDelegateKey):
            print 'The config file "' + \
                    configFilename + \
                    '" is missing the field "' + \
                    authDelegateKey + \
                    '".'
            sys.exit(1)
        else:
            authDelegateDict = configData[authDelegateKey]

# Check if all required keys are parsed.
requiredKeys = [CLIENT_ID, CLIENT_SECRET, REFRESH_TOKEN]
try:
    missingKey = requiredKeys[map(authDelegateDict.has_key,requiredKeys).index(False)];
    print 'Missing key: "' + missingKey + '". The list of required keys are:'
    print ' * ' + '\n * '.join(requiredKeys)
    print 'Exiting.'
    sys.exit(1)
except ValueError:
    pass

#get_refresh_token()
url += 'client_id=' + authDelegateDict[CLIENT_ID] + '&client_secret=' + authDelegateDict[CLIENT_SECRET]
defaultRefreshTokenString = authDelegateDict[REFRESH_TOKEN]
tokenRequest = requests.post(url, data = json.dumps(body), headers = headers)
if not tokenRequest.json().has_key('refresh_token'):
    print 'LWA did not return a refresh token, with the response code ' + \
            str(tokenRequest.status_code) + \
            '.  This is most likely due to an error. Check the following JSON response:<br/>' + \
            tokenRequest.text 
authDelegateDict['refreshToken'] = tokenRequest.json()['refresh_token']
try:
    configFile = open(configFilename,'r')
except IOError:
    print 'File "' + configFilename + '" cannot be opened!'
    sys.exit(1)
else:
    fileContent = configFile.read()
    try:
        configFile = open(configFilename,'w')
    except IOError:
        print 'File "' + configFilename + '" cannot be opened!'
        sys.exit(1)

    replaceWith = '\n\t\t\"refreshToken\":' + '\"' + tokenRequest.json()['refresh_token'] + '\",'
    stringToFind = r"\s*\"\s*{}\s*\"\s*:\s*\"\s*".format(REFRESH_TOKEN) + \
                    re.escape(defaultRefreshTokenString) + r"\s*\"\s*,"
    fileContent = re.sub(stringToFind, replaceWith, fileContent)
    configFile.write(fileContent)
    print 'Get REFRESH_TOKEN successfully.'
